@@ -10,7 +10,7 @@ from lightning_app.core.work import LightningWork
 from lightning_app.frontend import Frontend
 from lightning_app.storage import Path
 from lightning_app.storage.drive import _maybe_create_drive, Drive
-from lightning_app.utilities.app_helpers import _is_json_serializable, _LightningAppRef, _set_child_name
+from lightning_app.utilities.app_helpers import _is_json_serializable, _LightningAppRef, _set_child_name, is_overridden
 from lightning_app.utilities.component import _sanitize_state
 from lightning_app.utilities.exceptions import ExitAppException
 from lightning_app.utilities.introspection import _is_init_context, _is_run_context
@@ -142,6 +142,14 @@ class LightningFlow:
                 if name in self._works and value != getattr(self, name):
                     raise AttributeError(f"Cannot set attributes as the work can't be changed once defined: {name}")
 
+            if isinstance(value, (list, dict)) and value:
+                _type = (LightningFlow, LightningWork, List, Dict)
+                if isinstance(value, list) and all(isinstance(va, _type) for va in value):
+                    value = List(*value)
+
+                if isinstance(value, dict) and all(isinstance(va, _type) for va in value.values()):
+                    value = Dict(**value)
+
             if isinstance(value, LightningFlow):
                 self._flows.add(name)
                 _set_child_name(self, value, name)
@@ -163,10 +171,10 @@ class LightningFlow:
                 value._register_cloud_compute()
 
             elif isinstance(value, (Dict, List)):
-                value._backend = self._backend
                 self._structures.add(name)
                 _set_child_name(self, value, name)
-                if self._backend:
+                if getattr(self, "_backend", None) is not None:
+                    value._backend = self._backend
                     for flow in value.flows:
                         LightningFlow._attach_backend(flow, self._backend)
                     for work in value.works:
@@ -230,6 +238,15 @@ class LightningFlow:
             return Path.from_dict(self._paths[item])
         return self.__getattribute__(item)
 
+    @property
+    def ready(self) -> bool:
+        """Not currently enabled.
+
+        Override to customize when your App should be ready.
+        """
+        flows = self.flows
+        return all(flow.ready for flow in flows.values()) if flows else True
+
     @property
     def changes(self):
         return self._changes.copy()
@@ -757,8 +774,20 @@ class _RootFlow(LightningFlow):
         super().__init__()
         self.work = work
 
+    @property
+    def ready(self) -> bool:
+        ready = getattr(self.work, "ready", None)
+        if ready:
+            return ready
+        return self.work.url != ""
+
     def run(self):
+        if self.work.has_succeeded:
+            self.work.stop()
+            self._exit()
         self.work.run()
 
     def configure_layout(self):
-        return [{"name": "Main", "content": self.work}]
+        if is_overridden("configure_layout", self.work):
+            return [{"name": "Main", "content": self.work}]
+        return []
