@@ -1,29 +1,32 @@
 import hashlib
-import logging
 import pathlib
 import pickle
 from abc import ABC, abstractmethod
 from time import sleep
-from typing import Any, Optional, Union
+from typing import Any, Optional, TYPE_CHECKING, Union
 
-import lightning_app
+from lightning_app.core.constants import REMOTE_STORAGE_WAIT
 from lightning_app.core.queues import BaseQueue
-from lightning_app.storage.path import filesystem, Path, shared_storage_path
-from lightning_app.storage.requests import ExistsRequest, ExistsResponse, GetRequest, GetResponse
+from lightning_app.storage.path import _filesystem, _shared_storage_path, Path
+from lightning_app.storage.requests import _ExistsRequest, _ExistsResponse, _GetRequest, _GetResponse
+from lightning_app.utilities.app_helpers import Logger
 from lightning_app.utilities.component import _is_flow_context
 
-_logger = logging.getLogger(__name__)
+_logger = Logger(__name__)
+
+if TYPE_CHECKING:
+    from lightning_app.core.work import LightningWork
 
 
-class BasePayload(ABC):
+class _BasePayload(ABC):
     def __init__(self, value: Any) -> None:
         self._value = value
         # the attribute name given to the payload
         self._name: Optional[str] = None
         # the origin is the work that created this Path and wants to expose file(s)
-        self._origin: Optional[Union["lightning_app.LightningWork", str]] = None
+        self._origin: Optional[Union["LightningWork", str]] = None
         # the consumer is the Work that needs access to the file(s) from the consumer
-        self._consumer: Optional[Union["lightning_app.LightningWork", str]] = None
+        self._consumer: Optional[Union["LightningWork", str]] = None
         self._metadata = {}
         # request queue: used to transfer message to storage orchestrator
         self._request_queue: Optional[BaseQueue] = None
@@ -85,7 +88,7 @@ class BasePayload(ABC):
     def load(self, path: str) -> Any:
         """Override this method with your own loading logic."""
 
-    def _attach_work(self, work: "lightning_app.LightningWork") -> None:
+    def _attach_work(self, work: "LightningWork") -> None:
         """Attach a LightningWork to this PayLoad.
 
         Args:
@@ -122,11 +125,11 @@ class BasePayload(ABC):
             )
 
         # 1. Send message to orchestrator through queue that with a request for a path existence check
-        request = ExistsRequest(source=self.origin_name, name=self._name, path=str(self._path), hash=self.hash)
+        request = _ExistsRequest(source=self.origin_name, name=self._name, path=str(self._path), hash=self.hash)
         self._request_queue.put(request)
 
         # 2. Wait for the response to come back
-        response: ExistsResponse = self._response_queue.get()  # blocking
+        response: _ExistsResponse = self._response_queue.get()  # blocking
         return response.exists
 
     def get(self) -> Any:
@@ -147,29 +150,31 @@ class BasePayload(ABC):
         # 1. Send message to orchestrator through queue with details of the transfer
         # the source is the name of the work that owns the file that we request
         # the destination is determined by the queue, since each work has a dedicated send and recv queue
-        request = GetRequest(source=self.origin_name, name=self._name, path=str(self._path), hash=self.hash)
+        request = _GetRequest(source=self.origin_name, name=self._name, path=str(self._path), hash=self.hash)
         self._request_queue.put(request)
 
         # 2. Wait for the transfer to finish
-        response: GetResponse = self._response_queue.get()  # blocking
+        response: _GetResponse = self._response_queue.get()  # blocking
         self._validate_get_response(response)
 
-        fs = filesystem()
+        fs = _filesystem()
 
         # 3. Wait until the file appears in shared storage
-        while not fs.exists(response.path):
-            # TODO: Existence check on folder is not enough, files may not be completely transferred yet
-            sleep(0.5)
+        while not fs.exists(response.path) or fs.info(response.path)["size"] != response.size:
+            sleep(REMOTE_STORAGE_WAIT)
 
         # 4. Copy the file from the shared storage to the destination on the local filesystem
         local_path = self._path
         _logger.debug(f"Attempting to copy {str(response.path)} -> {str(local_path)}")
         fs.get(str(response.path), str(local_path), recursive=False)
 
+        # Ensure the file is properly written
+        sleep(0.5)
+
         self._value = self.load(local_path)
         return self._value
 
-    def _validate_get_response(self, response: "GetResponse") -> None:
+    def _validate_get_response(self, response: "_GetResponse") -> None:
         if response.source != self._origin or response.hash != self.hash:
             raise RuntimeError(
                 f"Tried to get the file {self} but received a response for a request it did not send. The response"
@@ -192,7 +197,7 @@ class BasePayload(ABC):
         )
 
     @classmethod
-    def from_dict(cls, content: dict) -> "BasePayload":
+    def from_dict(cls, content: dict) -> "_BasePayload":
         """Instantiate a Payload from a dictionary."""
         payload = cls(None)
         payload._name = content["name"]
@@ -202,8 +207,8 @@ class BasePayload(ABC):
         return payload
 
     @staticmethod
-    def _handle_exists_request(work: "lightning_app.LightningWork", request: ExistsRequest) -> ExistsResponse:
-        return ExistsResponse(
+    def _handle_exists_request(work: "LightningWork", request: _ExistsRequest) -> _ExistsResponse:
+        return _ExistsResponse(
             source=request.source,
             path=request.path,
             name=request.name,
@@ -213,12 +218,12 @@ class BasePayload(ABC):
         )
 
     @staticmethod
-    def _handle_get_request(work: "lightning_app.LightningWork", request: GetRequest) -> GetResponse:
-        from lightning_app.storage.copier import copy_files
+    def _handle_get_request(work: "LightningWork", request: _GetRequest) -> _GetResponse:
+        from lightning_app.storage.copier import _copy_files
 
         source_path = pathlib.Path(request.path)
-        destination_path = shared_storage_path() / request.hash
-        response = GetResponse(
+        destination_path = _shared_storage_path() / request.hash
+        response = _GetResponse(
             source=request.source,
             name=request.name,
             path=str(destination_path),
@@ -229,14 +234,15 @@ class BasePayload(ABC):
         try:
             payload = getattr(work, request.name)
             payload.save(payload.value, source_path)
-            copy_files(source_path, destination_path)
+            response.size = source_path.stat().st_size
+            _copy_files(source_path, destination_path)
             _logger.debug(f"All files copied from {request.path} to {response.path}.")
         except Exception as e:
             response.exception = e
         return response
 
 
-class Payload(BasePayload):
+class Payload(_BasePayload):
 
     """The Payload object enables to transfer python objects from one work to another in a similar fashion as
     :class:`~lightning_app.storage.path.Path`."""
