@@ -19,9 +19,11 @@ from unittest.mock import MagicMock
 
 import websockets
 from deepdiff import Delta
+from lightning_cloud.openapi import AppinstancesIdBody, Externalv1LightningappInstance
 
 import lightning_app
 from lightning_app.utilities.exceptions import LightningAppStateException
+from lightning_app.utilities.tree import breadth_first
 
 if TYPE_CHECKING:
     from lightning_app.core.app import LightningApp
@@ -128,13 +130,6 @@ class InMemoryStateStore(StateStore):
         self.store[k].session_id = v
 
 
-class DistributedMode(enum.Enum):
-    SINGLEPROCESS = enum.auto()
-    MULTIPROCESS = enum.auto()
-    CONTAINER = enum.auto()
-    GRID = enum.auto()
-
-
 class _LightningAppRef:
     _app_instance: Optional["LightningApp"] = None
 
@@ -516,14 +511,58 @@ def is_static_method(klass_or_instance, attr) -> bool:
     return isinstance(inspect.getattr_static(klass_or_instance, attr), staticmethod)
 
 
-def _debugger_is_active() -> bool:
-    """Return if the debugger is currently active."""
-    return hasattr(sys, "gettrace") and sys.gettrace() is not None
-
-
 def _should_dispatch_app() -> bool:
     return (
-        _debugger_is_active()
+        __debug__
+        and "_pytest.doctest" not in sys.modules
         and not bool(int(os.getenv("LIGHTNING_DISPATCHED", "0")))
         and "LIGHTNING_APP_STATE_URL" not in os.environ
     )
+
+
+def _is_headless(app: "LightningApp") -> bool:
+    """Utility which returns True if the given App has no ``Frontend`` objects or URLs exposed through
+    ``configure_layout``."""
+    if app.frontends:
+        return False
+    for component in breadth_first(app.root, types=(lightning_app.LightningFlow,)):
+        for entry in component._layout:
+            if "target" in entry:
+                return False
+    return True
+
+
+def _handle_is_headless(app: "LightningApp"):
+    """Utility for runtime-specific handling of changes to the ``is_headless`` property."""
+    app_id = os.getenv("LIGHTNING_CLOUD_APP_ID", None)
+    project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
+
+    if app_id is None or project_id is None:
+        return
+
+    from lightning_app.utilities.network import LightningClient
+
+    client = LightningClient()
+    list_apps_response = client.lightningapp_instance_service_list_lightningapp_instances(project_id=project_id)
+
+    current_lightningapp_instance: Optional[Externalv1LightningappInstance] = None
+    for lightningapp_instance in list_apps_response.lightningapps:
+        if lightningapp_instance.id == app_id:
+            current_lightningapp_instance = lightningapp_instance
+            break
+
+    if not current_lightningapp_instance:
+        raise RuntimeError(
+            "App was not found. Please open an issue at https://github.com/lightning-AI/lightning/issues."
+        )
+
+    if current_lightningapp_instance.spec.is_headless == app.is_headless:
+        return
+
+    current_lightningapp_instance.spec.is_headless = app.is_headless
+
+    client.lightningapp_instance_service_update_lightningapp_instance(
+        project_id=project_id,
+        id=current_lightningapp_instance.id,
+        body=AppinstancesIdBody(name=current_lightningapp_instance.name, spec=current_lightningapp_instance.spec),
+    )
